@@ -6,7 +6,7 @@
  * Felipe Borges <felipeborges@gnome.org>
  * Julian Sparber <julian@sparber.net>
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * SPDX-License-Identifier: LGPL-2.1+
  *
  */
 
@@ -21,29 +21,24 @@
 #define NUMBER_OF_COLORS 14
 #define LOAD_BUFFER_SIZE 65536
 /**
- * HdyAvatar:
- *
- * A widget displaying an image, with a generated fallback.
- *
- * `HdyAvatar` is a widget to display a round avatar.
+ * SECTION:hdy-avatar
+ * @short_description: A widget displaying an image, with a generated fallback.
+ * @Title: HdyAvatar
  *
+ * #HdyAvatar is a widget to display a round avatar.
  * A provided image is made round before displaying, if no image is given this
- * widget generates a round fallback with the initials of the
- * [property@Avatar:text] on top of a colored background.
+ * widget generates a round fallback with the initials of the #HdyAvatar:text
+ * on top of a colord background.
+ * The color is picked based on the hash of the #HdyAvatar:text.
+ * If #HdyAvatar:show-initials is set to %FALSE, `avatar-default-symbolic` is
+ * shown in place of the initials.
+ * Use hdy_avatar_set_loadable_icon() or #HdyAvatar:loadable-icon to set a
+ * custom image.
  *
- * The color is picked based on the hash of the [property@Avatar:text].
+ * # CSS nodes
  *
- * If [property@Avatar:show-initials] is set to `FALSE`,
- * `avatar-default-symbolic` is shown instead of the initials.
+ * #HdyAvatar has a single CSS node with name avatar.
  *
- * Use [method@Avatar.set_loadable_icon] or [property@Avatar:loadable-icon] to
- * set a custom image.
- *
- * ## CSS nodes
- *
- * `HdyAvatar` has a single CSS node with name `avatar`.
- *
- * Since: 1.0
  */
 
 struct _HdyAvatar
@@ -90,7 +85,7 @@ size_data_free (SizeData *data)
 }
 
 static void
-load_icon_async (HdyAvatar           *self,
+load_icon_async (GLoadableIcon       *icon,
                  gint                 size,
                  GCancellable        *cancellable,
                  GAsyncReadyCallback  callback,
@@ -149,21 +144,25 @@ extract_initials_from_text (const gchar *text)
   g_string_append_unichar (initials, unichar);
 
   q = g_utf8_strrchr (normalized, -1, ' ');
-  if (q != NULL) {
-    unichar = g_utf8_get_char (g_utf8_next_char (q));
+  if (q != NULL && g_utf8_next_char (q) != NULL) {
+    q = g_utf8_next_char (q);
 
-    if (unichar != 0)
-      g_string_append_unichar (initials, unichar);
+    unichar = g_utf8_get_char (q);
+    g_string_append_unichar (initials, unichar);
   }
 
   return g_string_free (initials, FALSE);
 }
 
 static GdkPixbuf *
-update_custom_image (GdkPixbuf *pixbuf_from_icon,
-                     GdkPixbuf *round_image,
-                     gint       new_size)
+update_custom_image (GLoadableIcon *icon,
+                     GdkPixbuf     *pixbuf_from_icon,
+                     GdkPixbuf     *round_image,
+                     gint           new_size)
 {
+  if (icon == NULL)
+    return NULL;
+
   if (round_image &&
       gdk_pixbuf_get_width (round_image) == new_size &&
       !is_scaled (round_image))
@@ -269,15 +268,12 @@ icon_load_async_cb (GLoadableIcon *icon,
                     GAsyncResult  *res,
                     GTask         *task)
 {
-  GdkPixbufLoader *loader = g_task_get_task_data (task);
   g_autoptr (GInputStream) stream = NULL;
   g_autoptr (GError) error = NULL;
 
   stream = g_loadable_icon_load_finish (icon, res, NULL, &error);
   if (stream == NULL) {
-    gdk_pixbuf_loader_close (loader, NULL);
     g_task_return_error (task, g_steal_pointer (&error));
-    g_object_unref (task);
 
     return;
   }
@@ -300,10 +296,11 @@ load_from_gicon_async_finish (GAsyncResult  *async_result,
 }
 
 static void
-load_from_gicon_async_for_display_cb (HdyAvatar    *self,
-                                      GAsyncResult *res,
-                                      gpointer     *user_data)
+load_from_gicon_async_for_display_cb (GLoadableIcon *icon,
+                                      GAsyncResult  *res,
+                                      gpointer      *user_data)
 {
+  HdyAvatar *self = HDY_AVATAR (user_data);
   g_autoptr (GError) error = NULL;
   g_autoptr (GdkPixbuf) pixbuf = NULL;
 
@@ -328,14 +325,13 @@ load_from_gicon_async_for_display_cb (HdyAvatar    *self,
     gint scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
     gint new_size = MIN (width, height) * scale_factor;
 
-    if (get_icon (self)) {
-      custom_image = update_custom_image (pixbuf,
-                                          NULL,
-                                          new_size);
+    custom_image = update_custom_image (icon,
+                                        pixbuf,
+                                        NULL,
+                                        new_size);
 
-      if (!self->round_image && custom_image)
-        gtk_style_context_add_class (context, "image");
-    }
+    if (!self->round_image && custom_image)
+      gtk_style_context_add_class (context, "image");
 
     g_set_object (&self->round_image, custom_image);
     gtk_widget_queue_draw (GTK_WIDGET (self));
@@ -343,9 +339,9 @@ load_from_gicon_async_for_display_cb (HdyAvatar    *self,
 }
 
 static void
-load_from_gicon_async_for_export_cb (HdyAvatar    *self,
-                                     GAsyncResult *res,
-                                     gpointer     *user_data)
+load_from_gicon_async_for_export_cb (GLoadableIcon *icon,
+                                     GAsyncResult  *res,
+                                     gpointer      *user_data)
 {
   GTask *task = G_TASK (user_data);
   g_autoptr (GError) error = NULL;
@@ -353,8 +349,7 @@ load_from_gicon_async_for_export_cb (HdyAvatar    *self,
 
   pixbuf = load_from_gicon_async_finish (res, &error);
 
-  if (error &&
-      !g_error_matches (error, HDY_AVATAR_ICON_ERROR, HDY_AVATAR_ICON_ERROR_EMPTY) &&
+  if (!g_error_matches (error, HDY_AVATAR_ICON_ERROR, HDY_AVATAR_ICON_ERROR_EMPTY) &&
       !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
     g_warning ("Failed to load icon: %s", error->message);
   }
@@ -362,17 +357,16 @@ load_from_gicon_async_for_export_cb (HdyAvatar    *self,
   g_task_return_pointer (task,
                          g_steal_pointer (&pixbuf),
                          g_object_unref);
-  g_object_unref (task);
 }
 
 static void
-load_icon_async (HdyAvatar           *self,
+load_icon_async (GLoadableIcon       *icon,
                  gint                 size,
                  GCancellable        *cancellable,
                  GAsyncReadyCallback  callback,
                  gpointer             user_data)
 {
-  GTask *task = g_task_new (self, cancellable, callback, user_data);
+  GTask *task = g_task_new (icon, cancellable, callback, user_data);
   GdkPixbufLoader *loader = gdk_pixbuf_loader_new ();
 
   g_signal_connect (loader, "size-prepared",
@@ -381,9 +375,7 @@ load_icon_async (HdyAvatar           *self,
 
   g_task_set_task_data (task, loader, g_object_unref);
 
-  g_loadable_icon_load_async (get_icon (self),
-                              size,
-                              cancellable,
+  g_loadable_icon_load_async (icon, size, cancellable,
                               (GAsyncReadyCallback) icon_load_async_cb,
                               task);
 }
@@ -627,7 +619,6 @@ hdy_avatar_dispose (GObject *object)
 
   g_cancellable_cancel (self->cancellable);
   g_clear_object (&self->icon);
-  g_clear_object (&self->load_func_icon);
 
   G_OBJECT_CLASS (hdy_avatar_parent_class)->dispose (object);
 }
@@ -733,7 +724,7 @@ hdy_avatar_draw (GtkWidget *widget,
   gint new_size = MIN (width, height) * scale_factor;
 
   if (get_icon (self)) {
-    custom_image = update_custom_image (NULL, self->round_image, new_size);
+    custom_image = update_custom_image (get_icon (self), NULL, self->round_image, new_size);
 
     if ((!custom_image &&
         !self->loading_error) ||
@@ -742,11 +733,11 @@ hdy_avatar_draw (GtkWidget *widget,
       self->currently_loading_size = new_size;
       g_cancellable_cancel (self->cancellable);
       g_set_object (&self->cancellable, g_cancellable_new ());
-      load_icon_async (self,
+      load_icon_async (get_icon (self),
                        new_size,
                        self->cancellable,
                        (GAsyncReadyCallback) load_from_gicon_async_for_display_cb,
-                       NULL);
+                       self);
     }
 
     /* We don't want to draw a broken custom image, because it may be scaled
@@ -827,6 +818,12 @@ hdy_avatar_get_preferred_height_for_width (GtkWidget *widget,
                       minimum, natural, NULL, NULL);
 }
 
+static GtkSizeRequestMode
+hdy_avatar_get_request_mode (GtkWidget *widget)
+{
+  return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
+}
+
 static void
 hdy_avatar_size_allocate (GtkWidget     *widget,
                           GtkAllocation *allocation)
@@ -859,6 +856,7 @@ hdy_avatar_class_init (HdyAvatarClass *klass)
   object_class->get_property = hdy_avatar_get_property;
 
   widget_class->draw = hdy_avatar_draw;
+  widget_class->get_request_mode = hdy_avatar_get_request_mode;
   widget_class->get_preferred_width = hdy_avatar_get_preferred_width;
   widget_class->get_preferred_height = hdy_avatar_get_preferred_height;
   widget_class->get_preferred_width_for_height = hdy_avatar_get_preferred_width_for_height;
@@ -866,11 +864,9 @@ hdy_avatar_class_init (HdyAvatarClass *klass)
   widget_class->size_allocate = hdy_avatar_size_allocate;
 
   /**
-   * HdyAvatar:size: (attributes org.gtk.Property.get=hdy_avatar_get_size org.gtk.Property.set=hdy_avatar_set_size)
+   * HdyAvatar:size:
    *
    * The avatar size of the avatar.
-   *
-   * Since: 1.0
    */
   props[PROP_SIZE] =
     g_param_spec_int ("size",
@@ -880,14 +876,14 @@ hdy_avatar_class_init (HdyAvatarClass *klass)
                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyAvatar:icon-name: (attributes org.gtk.Property.get=hdy_avatar_get_icon_name org.gtk.Property.set=hdy_avatar_set_icon_name)
+   * HdyAvatar:icon-name:
    *
-   * The name of an icon to use as a fallback.
-   *
-   * If no name is set, the avatar-default-symbolic icon will be used. If the
-   * name doesn't match a valid icon, it is an error and no icon will be
-   * displayed. If the icon theme is changed, the image will be updated
-   * automatically.
+   * The name of the icon in the icon theme to use when the icon should be
+   * displayed.
+   * If no name is set, the avatar-default-symbolic icon will be used.
+   * If the name doesn't match a valid icon, it is an error and no icon will be
+   * displayed.
+   * If the icon theme is changed, the image will be updated automatically.
    *
    * Since: 1.0
    */
@@ -899,14 +895,10 @@ hdy_avatar_class_init (HdyAvatarClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyAvatar:text: (attributes org.gtk.Property.get=hdy_avatar_get_text org.gtk.Property.set=hdy_avatar_set_text)
+   * HdyAvatar:text:
    *
-   * Sets the text used to generate the fallback initials and color.
-   *
-   * It's only used to generate the color if [property@Avatar:show-initials] is
-   * `FALSE`.
-   *
-   * Since: 1.0
+   * The text used for the initials and for generating the color.
+   * If #HdyAvatar:show-initials is %FALSE it's only used to generate the color.
    */
   props[PROP_TEXT] =
     g_param_spec_string ("text",
@@ -916,11 +908,9 @@ hdy_avatar_class_init (HdyAvatarClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyAvatar:show-initials: (attributes org.gtk.Property.get=hdy_avatar_get_show_initials org.gtk.Property.set=hdy_avatar_set_show_initials)
+   * HdyAvatar:show_initials:
    *
    * Whether to show the initials or the fallback icon on the generated avatar.
-   *
-   * Since: 1.0
    */
   props[PROP_SHOW_INITIALS] =
     g_param_spec_boolean ("show-initials",
@@ -930,9 +920,9 @@ hdy_avatar_class_init (HdyAvatarClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyAvatar:loadable-icon: (attributes org.gtk.Property.get=hdy_avatar_get_loadable_icon org.gtk.Property.set=hdy_avatar_set_loadable_icon)
+   * HdyAvatar:loadable-icon:
    *
-   * A [iface@Gio.LoadableIcon] used to load the avatar.
+   * A #GLoadableIcon used to load the avatar.
    *
    * Since: 1.2
    */
@@ -957,15 +947,15 @@ hdy_avatar_init (HdyAvatar *self)
 
 /**
  * hdy_avatar_new:
- * @size: the size of the avatar
- * @text: (nullable): the text used to get the initials and color
- * @show_initials: whether to use initials instead of an icon as fallback
+ * @size: The size of the avatar
+ * @text: (nullable): The text used to generate the color and initials if
+ * @show_initials is %TRUE. The color is selected at random if @text is empty.
+ * @show_initials: whether to show the initials or the fallback icon on
+ * top of the color generated based on @text.
  *
- * Creates a new `HdyAvatar`.
+ * Creates a new #HdyAvatar.
  *
- * Returns: the newly created `HdyAvatar`
- *
- * Since: 1.0
+ * Returns: the newly created #HdyAvatar
  */
 GtkWidget *
 hdy_avatar_new (gint         size,
@@ -980,12 +970,13 @@ hdy_avatar_new (gint         size,
 }
 
 /**
- * hdy_avatar_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
- * @self: an avatar
+ * hdy_avatar_get_icon_name:
+ * @self: a #HdyAvatar
  *
- * Gets the name of an icon to use as a fallback.
+ * Gets the name of the icon in the icon theme to use when the icon should be
+ * displayed.
  *
- * Returns: (nullable): the icon name
+ * Returns: (nullable) (transfer none): the name of the icon from the icon theme.
  *
  * Since: 1.0
  */
@@ -998,13 +989,16 @@ hdy_avatar_get_icon_name (HdyAvatar *self)
 }
 
 /**
- * hdy_avatar_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
- * @self: an avatar
+ * hdy_avatar_set_icon_name:
+ * @self: a #HdyAvatar
  * @icon_name: (nullable): the name of the icon from the icon theme
  *
- * Sets the name of an icon to use as a fallback.
- *
- * If no name is set, `avatar-default-symbolic` will be used.
+ * Sets the name of the icon in the icon theme to use when the icon should be
+ * displayed.
+ * If no name is set, the avatar-default-symbolic icon will be used.
+ * If the name doesn't match a valid icon, it is an error and no icon will be
+ * displayed.
+ * If the icon theme is changed, the image will be updated automatically.
  *
  * Since: 1.0
  */
@@ -1028,15 +1022,14 @@ hdy_avatar_set_icon_name (HdyAvatar   *self,
 }
 
 /**
- * hdy_avatar_get_text: (attributes org.gtk.Method.get_property=text)
- * @self: an avatar
+ * hdy_avatar_get_text:
+ * @self: a #HdyAvatar
  *
- * Gets the text used to generate the fallback initials and color.
+ * Get the text used to generate the fallback initials and color
  *
- * Returns: (nullable): the text used to generate the fallback initials and
- *   color
- *
- * Since: 1.0
+ * Returns: (nullable) (transfer none): returns the text used to generate
+ * the fallback initials. This is the internal string used by
+ * the #HdyAvatar, and must not be modified.
  */
 const gchar *
 hdy_avatar_get_text (HdyAvatar *self)
@@ -1047,13 +1040,11 @@ hdy_avatar_get_text (HdyAvatar *self)
 }
 
 /**
- * hdy_avatar_set_text: (attributes org.gtk.Method.set_property=text)
- * @self: an avatar
+ * hdy_avatar_set_text:
+ * @self: a #HdyAvatar
  * @text: (nullable): the text used to get the initials and color
  *
- * Set the text used to generate the fallback initials color.
- *
- * Since: 1.0
+ * Set the text used to generate the fallback initials color
  */
 void
 hdy_avatar_set_text (HdyAvatar   *self,
@@ -1075,14 +1066,12 @@ hdy_avatar_set_text (HdyAvatar   *self,
 }
 
 /**
- * hdy_avatar_get_show_initials: (attributes org.gtk.Method.get_property=show-initials)
- * @self: an avatar
+ * hdy_avatar_get_show_initials:
+ * @self: a #HdyAvatar
  *
- * Gets whether initials are used instead of an icon on the fallback avatar.
+ * Returns whether initials are used for the fallback or the icon.
  *
- * Returns: whether initials are used instead of an icon as fallback
- *
- * Since: 1.0
+ * Returns: %TRUE if the initials are used for the fallback.
  */
 gboolean
 hdy_avatar_get_show_initials (HdyAvatar *self)
@@ -1093,13 +1082,12 @@ hdy_avatar_get_show_initials (HdyAvatar *self)
 }
 
 /**
- * hdy_avatar_set_show_initials: (attributes org.gtk.Method.set_property=show-initials)
- * @self: an avatar
- * @show_initials: whether to use initials instead of an icon as fallback
+ * hdy_avatar_set_show_initials:
+ * @self: a #HdyAvatar
+ * @show_initials: whether the initials should be shown on the fallback avatar
+ * or the icon.
  *
- * Sets whether to use initials instead of an icon on the fallback avatar.
- *
- * Since: 1.0
+ * Sets whether the initials should be shown on the fallback avatar or the icon.
  */
 void
 hdy_avatar_set_show_initials (HdyAvatar *self,
@@ -1118,19 +1106,15 @@ hdy_avatar_set_show_initials (HdyAvatar *self,
 
 /**
  * hdy_avatar_set_image_load_func:
- * @self: an avatar
+ * @self: a #HdyAvatar
  * @load_image: (closure user_data) (nullable): callback to set a custom image
  * @user_data: (nullable): user data passed to @load_image
  * @destroy: (nullable): destroy notifier for @user_data
  *
- * A callback which is called when the custom image needs to be reloaded.
+ * A callback which is called when the custom image need to be reloaded for some
+ * reason (e.g. scale-factor changes).
  *
- * It will be called on [property@Avatar:size] or
- * [property@Gtk.Widget:scale-factor] changes.
- *
- * Since: 1.0
- *
- * Deprecated: 1.2: use [method@Avatar.set_loadable_icon] instead.
+ * Deprecated: 1.2: use hdy_avatar_set_loadable_icon() instead.
  */
 void
 hdy_avatar_set_image_load_func (HdyAvatar              *self,
@@ -1164,25 +1148,23 @@ hdy_avatar_set_image_load_func (HdyAvatar              *self,
 
     self->cancellable = g_cancellable_new ();
     self->currently_loading_size = self->size * scale_factor;
-    load_icon_async (self,
+    load_icon_async (G_LOADABLE_ICON (self->load_func_icon),
                      self->currently_loading_size,
                      self->cancellable,
                      (GAsyncReadyCallback) load_from_gicon_async_for_display_cb,
-                     NULL);
+                     self);
   } else {
     gtk_widget_queue_draw (GTK_WIDGET (self));
   }
 }
 
 /**
- * hdy_avatar_get_size: (attributes org.gtk.Method.get_property=size)
- * @self: an avatar
+ * hdy_avatar_get_size:
+ * @self: a #HdyAvatar
  *
- * Gets the size of the avatar.
+ * Returns the size of the avatar.
  *
- * Returns: the size of the avatar
- *
- * Since: 1.0
+ * Returns: the size of the avatar.
  */
 gint
 hdy_avatar_get_size (HdyAvatar *self)
@@ -1193,13 +1175,11 @@ hdy_avatar_get_size (HdyAvatar *self)
 }
 
 /**
- * hdy_avatar_set_size: (attributes org.gtk.Method.set_property=size)
- * @self: an avatar
- * @size: the size to be used for the avatar
+ * hdy_avatar_set_size:
+ * @self: a #HdyAvatar
+ * @size: The size to be used for the avatar
  *
  * Sets the size of the avatar.
- *
- * Since: 1.0
  */
 void
 hdy_avatar_set_size (HdyAvatar *self,
@@ -1219,15 +1199,13 @@ hdy_avatar_set_size (HdyAvatar *self,
 
 /**
  * hdy_avatar_draw_to_pixbuf:
- * @self: an avatar
- * @size: the size of the pixbuf
- * @scale_factor: the scale factor
+ * @self: a #HdyAvatar
+ * @size: The size of the pixbuf
+ * @scale_factor: The scale factor
  *
- * Renders @self into a [class@GdkPixbuf.Pixbuf] at @size and @scale_factor.
+ * Renders @self into a pixbuf at @size and @scale_factor. This can be used to export the fallback avatar.
  *
- * This can be used to export the fallback avatar.
- *
- * Returns: (transfer full): the pixbuf
+ * Returns: (transfer full): the pixbuf.
  *
  * Since: 1.2
  */
@@ -1265,10 +1243,9 @@ hdy_avatar_draw_to_pixbuf (HdyAvatar *self,
         is_scaled (self->round_image) ||
         gdk_pixbuf_get_width (self->round_image) != scaled_size) {
       pixbuf_from_icon = load_icon_sync (get_icon (self), scaled_size);
-      custom_image = update_custom_image (pixbuf_from_icon, NULL, scaled_size);
-      gtk_style_context_add_class (context, "image");
+      custom_image = update_custom_image (get_icon (self), pixbuf_from_icon, NULL, scaled_size);
     } else {
-      custom_image = update_custom_image (NULL, self->round_image, scaled_size);
+      custom_image = update_custom_image (get_icon (self), NULL, self->round_image, scaled_size);
     }
   }
 
@@ -1281,16 +1258,14 @@ hdy_avatar_draw_to_pixbuf (HdyAvatar *self,
 
 /**
  * hdy_avatar_draw_to_pixbuf_async:
- * @self: an avatar
- * @size: the size of the pixbuf
- * @scale_factor: the scale factor
- * @cancellable: (nullable): a cancellable
- * @callback: (scope async): a [callback@Gio.AsyncReadyCallback] to call when
- *   the avatar is generated
+ * @self: a #HdyAvatar
+ * @size: The size of the pixbuf
+ * @scale_factor: The scale factor
+ * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore
+ * @callback: (scope async): a #GAsyncReadyCallback to call when the avatar is generated
  * @user_data: (closure): the data to pass to callback function
 
  * Renders asynchronously @self into a pixbuf at @size and @scale_factor.
- *
  * This can be used to export the fallback avatar.
  *
  * Since: 1.2
@@ -1303,7 +1278,7 @@ hdy_avatar_draw_to_pixbuf_async (HdyAvatar           *self,
                                  GAsyncReadyCallback  callback,
                                  gpointer             user_data)
 {
-  g_autoptr (GTask) task = NULL;
+  GTask *task = NULL;
   gint scaled_size = size * scale_factor;
   SizeData *data;
 
@@ -1323,23 +1298,23 @@ hdy_avatar_draw_to_pixbuf_async (HdyAvatar           *self,
       (!self->round_image ||
        gdk_pixbuf_get_width (self->round_image) != scaled_size ||
        is_scaled (self->round_image)))
-    load_icon_async (self,
+    load_icon_async (get_icon (self),
                      scaled_size,
                      cancellable,
                      (GAsyncReadyCallback) load_from_gicon_async_for_export_cb,
-                     g_steal_pointer (&task));
+                     task);
   else
     g_task_return_pointer (task, NULL, NULL);
 }
 
 /**
  * hdy_avatar_draw_to_pixbuf_finish:
- * @self: an avatar
- * @async_result: a [iface@Gio.AsyncResult]
+ * @self: a #HdyAvatar
+ * @async_result: a #GAsyncResult
  *
  * Finishes an asynchronous draw of an avatar to a pixbuf.
  *
- * Returns: (transfer full): the resulting pixbuf
+ * Returns: (transfer full): a #GdkPixbuf
  *
  * Since: 1.2
  */
@@ -1376,7 +1351,8 @@ hdy_avatar_draw_to_pixbuf_finish (HdyAvatar    *self,
   cairo_translate (cr, -bounds.x, -bounds.y);
 
   pixbuf_from_icon = g_task_propagate_pointer (task, NULL);
-  custom_image = update_custom_image (pixbuf_from_icon,
+  custom_image = update_custom_image (get_icon (self),
+                                      pixbuf_from_icon,
                                       NULL,
                                       data->size * data->scale_factor);
   draw_for_size (self, cr, custom_image, data->size, data->size, data->scale_factor);
@@ -1387,12 +1363,12 @@ hdy_avatar_draw_to_pixbuf_finish (HdyAvatar    *self,
 }
 
 /**
- * hdy_avatar_get_loadable_icon: (attributes org.gtk.Method.get_property=loadable-icon)
- * @self: an avatar
+ * hdy_avatar_get_loadable_icon:
+ * @self: a #HdyAvatar
  *
- * Gets the [iface@Gio.LoadableIcon] set via [method@Avatar.set_loadable_icon].
+ * Gets the #GLoadableIcon set via hdy_avatar_set_loadable_icon().
  *
- * Returns: (nullable) (transfer none): the [iface@Gio.LoadableIcon]
+ * Returns: (nullable) (transfer none): the #GLoadableIcon
  *
  * Since: 1.2
  */
@@ -1405,17 +1381,15 @@ hdy_avatar_get_loadable_icon (HdyAvatar *self)
 }
 
 /**
- * hdy_avatar_set_loadable_icon: (attributes org.gtk.Method.set_property=loadable-icon)
- * @self: an avatar
- * @icon: (nullable): a [iface@Gio.LoadableIcon]
+ * hdy_avatar_set_loadable_icon:
+ * @self: a #HdyAvatar
+ * @icon: (nullable): a #GLoadableIcon
  *
- * Sets the [iface@Gio.LoadableIcon] to use as an avatar.
+ * Sets the #GLoadableIcon to use as an avatar.
+ * The previous avatar is displayed till the new avatar is loaded,
+ * to immediately remove the custom avatar set the loadable-icon to %NULL.
  *
- * The previous avatar is displayed till the new avatar is loaded, to
- * immediately remove the custom avatar set the loadable-icon to `NULL`.
- *
- * The [iface@Gio.LoadableIcon] set via this function is preferred over a set
- * [callback@AvatarImageLoadFunc].
+ * The #GLoadableIcon set via this function is prefered over a set #HdyAvatarImageLoadFunc.
  *
  * Since: 1.2
  */
@@ -1441,11 +1415,11 @@ hdy_avatar_set_loadable_icon (HdyAvatar     *self,
   if (self->icon) {
     gint scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (self));
     self->currently_loading_size = self->size * scale_factor;
-    load_icon_async (self,
+    load_icon_async (self->icon,
                      self->currently_loading_size,
                      self->cancellable,
                      (GAsyncReadyCallback) load_from_gicon_async_for_display_cb,
-                     NULL);
+                     self);
   } else {
     gtk_widget_queue_draw (GTK_WIDGET (self));
   }
