@@ -2,7 +2,7 @@
  * Copyright (C) 2020 Felix Häcker <haeckerfelix@gnome.org>
  * Copyright (C) 2020 Purism SPC
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 #include "config.h"
@@ -17,55 +17,51 @@
 #include "hdy-swipe-tracker-private.h"
 
 /**
- * HdyFlap:
+ * SECTION:hdy-flap
+ * @short_description: An adaptive container acting like a box or an overlay.
+ * @Title: HdyFlap
  *
- * An adaptive container acting like a box or an overlay.
+ * The #HdyFlap widget can display its children like a #GtkBox does or like a
+ * #GtkOverlay does, according to the #HdyFlap:fold-policy value.
  *
- * The `HdyFlap` widget can display its children like a [class@Gtk.Box] does or
- * like a [class@Gtk.Overlay] does, according to the
- * [property@Flap:fold-policy] value.
+ * #HdyFlap has at most three children: #HdyFlap:content, #HdyFlap:flap and
+ * #HdyFlap:separator. Content is the primary child, flap is displayed next to
+ * it when unfolded, or overlays it when folded. Flap can be shown or hidden by
+ * changing the #HdyFlap:reveal-flap value, as well as via swipe gestures if
+ * #HdyFlap:swipe-to-open and/or #HdyFlap:swipe-to-close are set to %TRUE.
  *
- * `HdyFlap` has at most three children: [property@Flap:content],
- * [property@Flap:flap] and [property@Flap:separator]. Content is the primary
- * child, flap is displayed next to it when unfolded, or overlays it when
- * folded. Flap can be shown or hidden by changing the
- * [property@Flap:reveal-flap] value, as well as via swipe gestures if
- * [property@Flap:swipe-to-open] and/or [property@Flap:swipe-to-close] are set
- * to `TRUE`.
+ * Optionally, a separator can be provided, which would be displayed between
+ * the content and the flap when there's no shadow to separate them, depending
+ * on the transition type.
  *
- * Optionally, a separator can be provided, which would be displayed between the
- * content and the flap when there's no shadow to separate them, depending on
- * the transition type.
+ * #HdyFlap:flap is transparent by default; add the .background style class to
+ * it if this is unwanted.
  *
- * [property@Flap:flap] is transparent by default; add the `.background` style
- * class to it if this is unwanted.
- *
- * If [property@Flap:modal] is set to `TRUE`, content becomes completely
- * inaccessible when the flap is revealed while folded.
+ * If #HdyFlap:modal is set to %TRUE, content becomes completely inaccessible
+ * when the flap is revealed when folded.
  *
  * The position of the flap and separator children relative to the content is
- * determined by orientation, as well as the [property@Flap:flap-position]
- * value.
+ * determined by orientation, as well as  #HdyFlap:flap-position value.
  *
  * Folding the flap will automatically hide the flap widget, and unfolding it
  * will automatically reveal it. If this behavior is not desired, the
- * [property@Flap:locked] property can be used to override it.
+ * #HdyFlap:locked property can be used to override it.
  *
  * Common use cases include sidebars, header bars that need to be able to
  * overlap the window content (for example, in fullscreen mode) and bottom
  * sheets.
  *
- * ## HdyFlap as GtkBuildable
+ * # HdyFlap as GtkBuildable
  *
- * The `HdyFlap` implementation of the [iface@Gtk.Buildable] interface supports
- * setting the flap child by specifying “flap” as the “type” attribute of a
- * <child> element, and separator by specifying “separator”. Specifying
+ * The #HdyFlap implementation of the #GtkBuildable interface supports setting
+ * the flap child by specifying “flap” as the “type” attribute of a
+ * &lt;child&gt; element, and separator by specifying “separator”. Specifying
  * “content” child type or omitting it results in setting the content child.
  *
- * ## CSS nodes
+ * # CSS nodes
  *
- * `HdyFlap` has a single CSS node with name `flap`. The node will get the style
- * classes `.folded` when it is folded, and `.unfolded` when it's not.
+ * #HdyFlap has a single CSS node with name flap. The node will get the style
+ * classes .folded when it is folded, and .unfolded when it's not.
  *
  * Since: 1.2
  */
@@ -78,7 +74,8 @@
  * @HDY_FLAP_FOLD_POLICY_AUTO: Fold and unfold the flap based on available
  *   space.
  *
- * Describes the possible folding behavior of a [class@Flap] widget.
+ * These enumeration values describe the possible folding behavior in a #HdyFlap
+ * widget.
  *
  * Since: 1.2
  */
@@ -93,11 +90,9 @@
  *   neither the flap nor content overlap each other. Both widgets can be
  *   swiped.
  *
- * Describes transitions types of a [class@Flap] widget.
- *
  * These enumeration values describe the possible transitions between children
- * in a [class@Flap] widget, as well as which areas can be swiped via
- * [property@Flap:swipe-to-open] and [property@Flap:swipe-to-close].
+ * in a #HdyFlap widget, as well as which areas can be swiped via
+ * #HdyFlap:swipe-to-open and #HdyFlap:swipe-to-close.
  *
  * New values may be added to this enum over time.
  *
@@ -360,19 +355,6 @@ set_reveal_flap (HdyFlap  *self,
       hdy_swipeable_emit_child_switched (HDY_SWIPEABLE (self), reveal_flap ? 1 : 0, duration);
   }
 
-  if (self->reveal_flap &&
-      self->content.widget &&
-      self->flap.widget &&
-      self->modal &&
-      self->fold_progress > 0 &&
-      gtk_widget_get_mapped (GTK_WIDGET (self))) {
-    GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
-    GtkWidget *focus = gtk_window_get_focus (GTK_WINDOW (toplevel));
-
-    if (focus && gtk_widget_is_ancestor (focus, self->content.widget))
-      gtk_widget_child_focus (GTK_WIDGET (self), GTK_DIR_TAB_FORWARD);
-  }
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_REVEAL_FLAP]);
 }
 
@@ -758,6 +740,7 @@ compute_sizes (HdyFlap       *self,
 
   if (extra > 0 && flap_expand) {
     *flap_size += extra;
+    extra = 0;
 
     if (!revealed)
       *content_size = total;
@@ -1001,7 +984,6 @@ hdy_flap_size_allocate (GtkWidget     *widget,
                   self->fold_progress > 0);
 
   gtk_widget_set_clip (widget, alloc);
-  gtk_widget_queue_draw (widget);
 }
 
 /* This private method is prefixed by the call name because it will be a virtual
@@ -1255,7 +1237,6 @@ hdy_flap_realize (GtkWidget *widget)
   register_window (self, &self->separator);
   register_window (self, &self->flap);
 
-  update_child_visibility (self);
   restack_windows (self);
 }
 
@@ -1283,30 +1264,6 @@ hdy_flap_direction_changed (GtkWidget        *widget,
                                                                previous_direction);
 }
 
-static gboolean
-hdy_flap_focus (GtkWidget        *widget,
-                GtkDirectionType  direction)
-{
-  HdyFlap *self = HDY_FLAP (widget);
-
-  if (!gtk_widget_get_can_focus (widget) &&
-      self->content.widget &&
-      self->flap.widget &&
-      self->modal &&
-      self->reveal_progress > 0 &&
-      self->fold_progress > 0) {
-    if (gtk_widget_child_focus (GTK_WIDGET (self->flap.widget), direction))
-      return GDK_EVENT_STOP;
-
-    if (self->separator.widget)
-      return gtk_widget_child_focus (GTK_WIDGET (self->separator.widget), direction);
-
-    return GDK_EVENT_PROPAGATE;
-  }
-
-  return GTK_WIDGET_CLASS (hdy_flap_parent_class)->focus (widget, direction);
-}
-
 static void
 hdy_flap_forall (GtkContainer *container,
                  gboolean      include_internals,
@@ -1513,18 +1470,15 @@ hdy_flap_class_init (HdyFlapClass *klass)
   widget_class->realize = hdy_flap_realize;
   widget_class->unrealize = hdy_flap_unrealize;
   widget_class->direction_changed = hdy_flap_direction_changed;
-  widget_class->focus = hdy_flap_focus;
 
   container_class->remove = hdy_flap_remove;
   container_class->add = hdy_flap_add;
   container_class->forall = hdy_flap_forall;
 
   /**
-   * HdyFlap:content: (attributes org.gtk.Property.get=hdy_flap_get_content org.gtk.Property.set=hdy_flap_set_content)
+   * HdyFlap:content:
    *
-   * The content widget.
-   *
-   * It's always displayed when unfolded, and partially visible
+   * The content widget, always displayed when unfolded, and partially visible
    * when folded.
    *
    * Since: 1.2
@@ -1537,11 +1491,10 @@ hdy_flap_class_init (HdyFlapClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:flap: (attributes org.gtk.Property.get=hdy_flap_get_flap org.gtk.Property.set=hdy_flap_set_flap)
+   * HdyFlap:flap:
    *
-   * The flap widget.
-   *
-   * It's only visible when [property@Flap:reveal-progress] is greater than 0.
+   * The flap widget, only visible when #HdyFlap:reveal-progress is greater than
+   * 0.
    *
    * Since: 1.2
    */
@@ -1553,13 +1506,11 @@ hdy_flap_class_init (HdyFlapClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:separator: (attributes org.gtk.Property.get=hdy_flap_get_separator org.gtk.Property.set=hdy_flap_set_separator)
+   * HdyFlap:separator:
    *
-   * The separator widget.
-   *
-   * It's displayed between content and flap when there's no shadow to display.
-   * When exactly it's visible depends on the [property@Flap:transition-type]
-   * value. If `NULL`, no separator will be used.
+   * The separator widget, displayed between content and flap when there's no
+   * shadow to display. When exactly it's visible depends on the
+   * #HdyFlap:transition-type value. If %NULL, no separator will be used.
    *
    * Since: 1.2
    */
@@ -1571,12 +1522,10 @@ hdy_flap_class_init (HdyFlapClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:flap-position: (attributes org.gtk.Property.get=hdy_flap_get_flap_position org.gtk.Property.set=hdy_flap_set_flap_position)
+   * HdyFlap:flap-position:
    *
-   * The flap position.
-   *
-   * If `GTK_PACK_START`, the flap is displayed before the content, if
-   * `GTK_PACK_END`, it's displayed after the content.
+   * The flap position for @self. If @GTK_PACK_START, the flap is displayed
+   * before the content, if @GTK_PACK_END, it's displayed after the content.
    *
    * Since: 1.2
    */
@@ -1589,7 +1538,7 @@ hdy_flap_class_init (HdyFlapClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:reveal-flap: (attributes org.gtk.Property.get=hdy_flap_get_reveal_flap org.gtk.Property.set=hdy_flap_set_reveal_flap)
+   * HdyFlap:reveal-flap:
    *
    * Whether the flap widget is revealed.
    *
@@ -1603,7 +1552,7 @@ hdy_flap_class_init (HdyFlapClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:reveal-duration: (attributes org.gtk.Property.get=hdy_flap_get_reveal_duration org.gtk.Property.set=hdy_flap_set_reveal_duration)
+   * HdyFlap:reveal-duration:
    *
    * The reveal transition animation duration, in milliseconds.
    *
@@ -1618,12 +1567,10 @@ hdy_flap_class_init (HdyFlapClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:reveal-progress: (attributes org.gtk.Property.get=hdy_flap_get_reveal_progress)
+   * HdyFlap:reveal-progress:
    *
-   * The current reveal transition progress.
-   *
-   * 0 means fully hidden, 1 means fully revealed. See
-   * [property@Flap:reveal-flap].
+   * The current reveal transition progress. 0 means fully hidden, 1 means fully
+   * revealed See #HdyFlap:reveal-flap.
    *
    * Since: 1.2
    */
@@ -1635,11 +1582,10 @@ hdy_flap_class_init (HdyFlapClass *klass)
                           G_PARAM_READABLE);
 
   /**
-   * HdyFlap:fold-policy: (attributes org.gtk.Property.get=hdy_flap_get_fold_policy org.gtk.Property.set=hdy_flap_set_fold_policy)
+   * HdyFlap:fold-policy:
    *
-   * The current fold policy.
-   *
-   * See [enum@FlapFoldPolicy] for available policies.
+   * The current fold policy. See #HdyFlapFoldPolicy for available
+   * policies.
    *
    * Since: 1.2
    */
@@ -1652,7 +1598,7 @@ hdy_flap_class_init (HdyFlapClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:fold-duration: (attributes org.gtk.Property.get=hdy_flap_get_fold_duration org.gtk.Property.set=hdy_flap_set_fold_duration)
+   * HdyFlap:fold-duration:
    *
    * The fold transition animation duration, in milliseconds.
    *
@@ -1667,11 +1613,11 @@ hdy_flap_class_init (HdyFlapClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:folded: (attributes org.gtk.Property.get=hdy_flap_get_folded)
+   * HdyFlap:folded:
    *
    * Whether the flap is currently folded.
    *
-   * See [property@Flap:fold-policy].
+   * See #HdyFlap:fold-policy.
    *
    * Since: 1.2
    */
@@ -1683,13 +1629,13 @@ hdy_flap_class_init (HdyFlapClass *klass)
                           G_PARAM_READABLE);
 
   /**
-   * HdyFlap:locked: (attributes org.gtk.Property.get=hdy_flap_get_locked org.gtk.Property.set=hdy_flap_set_locked)
+   * HdyFlap:locked:
    *
    * Whether the flap is locked.
    *
-   * If `FALSE`, folding when the flap is revealed automatically closes it, and
-   * unfolding it when the flap is not revealed opens it. If `TRUE`,
-   * [property@Flap:reveal-flap] value never changes on its own.
+   * If %FALSE, folding when the flap is revealed automatically closes it, and
+   * unfolding it when the flap is not revealed opens it. If %TRUE,
+   * #HdyFlap:reveal-flap value never changes on its own.
    *
    * Since: 1.2
    */
@@ -1701,13 +1647,14 @@ hdy_flap_class_init (HdyFlapClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:transition-type: (attributes org.gtk.Property.get=hdy_flap_get_transition_type org.gtk.Property.set=hdy_flap_set_transition_type)
+   * HdyFlap:transition-type:
    *
-   * the type of animation used for reveal and fold transitions.
+   * The type of animation that will be used for reveal and fold transitions
+   * in @self.
    *
-   * [property@Flap:flap] is transparent by default, which means the content
-   * will be seen through it with `HDY_FLAP_TRANSITION_TYPE_OVER` transitions;
-   * add the `.background` style class to it if this is unwanted.
+   * #HdyFlap:flap is transparent by default, which means the content will be
+   * seen through it with %HDY_FLAP_TRANSITION_TYPE_OVER transitions; add the
+   * .background style class to it if this is unwanted.
    *
    * Since: 1.2
    */
@@ -1720,13 +1667,13 @@ hdy_flap_class_init (HdyFlapClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:modal: (attributes org.gtk.Property.get=hdy_flap_get_modal org.gtk.Property.set=hdy_flap_set_modal)
+   * HdyFlap:modal:
    *
    * Whether the flap is modal.
    *
-   * If `TRUE`, clicking the content widget while flap is revealed, as well as
-   * pressing the <kbd>Esc</kbd> key, will close the flap. If `FALSE`, clicks
-   * are passed through to the content widget.
+   * If %TRUE, clicking the content widget while flap is revealed, as well as
+   * pressing Escape key, will close the flap. If %FALSE, clicks are passed
+   * through to the content widget.
    *
    * Since: 1.2
    */
@@ -1738,12 +1685,11 @@ hdy_flap_class_init (HdyFlapClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:swipe-to-open: (attributes org.gtk.Property.get=hdy_flap_get_swipe_to_open org.gtk.Property.set=hdy_flap_set_swipe_to_open)
+   * HdyFlap:swipe-to-open:
    *
    * Whether the flap can be opened with a swipe gesture.
    *
-   * The area that can be swiped depends on the [property@Flap:transition-type]
-   * value.
+   * The area that can be swiped depends on the #HdyFlap:transition-type value.
    *
    * Since: 1.2
    */
@@ -1755,12 +1701,11 @@ hdy_flap_class_init (HdyFlapClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * HdyFlap:swipe-to-close: (attributes org.gtk.Property.get=hdy_flap_get_swipe_to_close org.gtk.Property.set=hdy_flap_set_swipe_to_close)
+   * HdyFlap:swipe-to-close:
    *
    * Whether the flap can be closed with a swipe gesture.
    *
-   * The area that can be swiped depends on the [property@Flap:transition-type]
-   * value.
+   * The area that can be swiped depends on the #HdyFlap:transition-type value.
    *
    * Since: 1.2
    */
@@ -2027,9 +1972,9 @@ hdy_flap_swipeable_init (HdySwipeableInterface *iface)
 /**
  * hdy_flap_new:
  *
- * Creates a new `HdyFlap`.
+ * Creates a new #HdyFlap.
  *
- * Returns: the newly created `HdyFlap`
+ * Returns: a new #HdyFlap
  *
  * Since: 1.2
  */
@@ -2040,8 +1985,8 @@ hdy_flap_new (void)
 }
 
 /**
- * hdy_flap_get_content: (attributes org.gtk.Method.get_property=content)
- * @self: a flap
+ * hdy_flap_get_content:
+ * @self: a #HdyFlap
  *
  * Gets the content widget for @self
  *
@@ -2058,13 +2003,12 @@ hdy_flap_get_content (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_content: (attributes org.gtk.Method.set_property=content)
- * @self: a flap
- * @content: (nullable): the content widget
+ * hdy_flap_set_content:
+ * @self: a #HdyFlap
+ * @content: (nullable): the content widget, or %NULL
  *
- * Sets the content widget for @self.
- *
- * It is always displayed when unfolded, and partially visible when folded.
+ * Sets the content widget for @self, always displayed when unfolded, and
+ * partially visible when folded.
  *
  * Since: 1.2
  */
@@ -2092,8 +2036,8 @@ hdy_flap_set_content (HdyFlap   *self,
 }
 
 /**
- * hdy_flap_get_flap: (attributes org.gtk.Method.get_property=flap)
- * @self: a flap
+ * hdy_flap_get_flap:
+ * @self: a #HdyFlap
  *
  * Gets the flap widget for @self
  *
@@ -2110,11 +2054,12 @@ hdy_flap_get_flap (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_flap: (attributes org.gtk.Method.set_property=flap)
- * @self: a flap
- * @flap: (nullable): the flap widget
+ * hdy_flap_set_flap:
+ * @self: a #HdyFlap
+ * @flap: (nullable): the flap widget, or %NULL
  *
- * Sets the flap widget for @self.
+ * Sets the flap widget for @self, only visible when #HdyFlap:reveal-progress is
+ * greater than 0.
  *
  * Since: 1.2
  */
@@ -2143,8 +2088,8 @@ hdy_flap_set_flap (HdyFlap   *self,
 }
 
 /**
- * hdy_flap_get_separator: (attributes org.gtk.Method.get_property=separator)
- * @self: a flap
+ * hdy_flap_get_separator:
+ * @self: a #HdyFlap
  *
  * Gets the separator widget for @self.
  *
@@ -2161,11 +2106,13 @@ hdy_flap_get_separator (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_separator: (attributes org.gtk.Method.set_property=separator)
- * @self: a flap
- * @separator: (nullable): the separator widget
+ * hdy_flap_set_separator:
+ * @self: a #HdyFlap
+ * @separator: (nullable): the separator widget, or %NULL
  *
- * Sets the separator widget for @self.
+ * Sets the separator widget for @self, displayed between content and flap when
+ * there's no shadow to display. When exactly it's visible depends on the
+ * #HdyFlap:transition-type value. If %NULL, no separator will be used.
  *
  * Since: 1.2
  */
@@ -2193,8 +2140,8 @@ hdy_flap_set_separator (HdyFlap   *self,
 }
 
 /**
- * hdy_flap_get_flap_position: (attributes org.gtk.Method.get_property=flap-position)
- * @self: a flap
+ * hdy_flap_get_flap_position:
+ * @self: a #HdyFlap
  *
  * Gets the flap position for @self.
  *
@@ -2211,11 +2158,12 @@ hdy_flap_get_flap_position (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_flap_position: (attributes org.gtk.Method.set_property=flap-position)
- * @self: a flap
+ * hdy_flap_set_flap_position:
+ * @self: a #HdyFlap
  * @position: the new value
  *
- * Sets the flap position for @self.
+ * Sets the flap position for @self. If @GTK_PACK_START, the flap is displayed
+ * before the content, if @GTK_PACK_END, it's displayed after the content.
  *
  * Since: 1.2
  */
@@ -2239,12 +2187,12 @@ hdy_flap_set_flap_position (HdyFlap     *self,
 }
 
 /**
- * hdy_flap_get_reveal_flap: (attributes org.gtk.Method.get_property=reveal-flap)
- * @self: a flap
+ * hdy_flap_get_reveal_flap:
+ * @self: a #HdyFlap
  *
  * Gets whether the flap widget is revealed for @self.
  *
- * Returns: whether flap widget is revealed
+ * Returns: %TRUE if the flap widget is revealed, %FALSE otherwise.
  *
  * Since: 1.2
  */
@@ -2257,9 +2205,9 @@ hdy_flap_get_reveal_flap (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_reveal_flap: (attributes org.gtk.Method.set_property=reveal-flap)
- * @self: a flap
- * @reveal_flap: `TRUE` to reveal the flap widget, `FALSE` otherwise
+ * hdy_flap_set_reveal_flap:
+ * @self: a #HdyFlap
+ * @reveal_flap: %TRUE to reveal the flap widget, %FALSE otherwise
  *
  * Sets whether the flap widget is revealed for @self.
  *
@@ -2275,12 +2223,13 @@ hdy_flap_set_reveal_flap (HdyFlap  *self,
 }
 
 /**
- * hdy_flap_get_reveal_duration: (attributes org.gtk.Method.get_property=reveal-duration)
- * @self: a flap
+ * hdy_flap_get_reveal_duration:
+ * @self: a #HdyFlap
  *
- * Gets the amount of time that reveal transitions will take.
+ * Returns the amount of time (in milliseconds) that reveal transitions in @self
+ * will take.
  *
- * Returns: the reveal transition duration, in milliseconds
+ * Returns: the reveal transition duration
  *
  * Since: 1.2
  */
@@ -2293,8 +2242,8 @@ hdy_flap_get_reveal_duration (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_reveal_duration: (attributes org.gtk.Method.set_property=reveal-duration)
- * @self: a flap
+ * hdy_flap_set_reveal_duration:
+ * @self: a #HdyFlap
  * @duration: the new duration, in milliseconds
  *
  * Sets the duration that reveal transitions in @self will take.
@@ -2316,10 +2265,11 @@ hdy_flap_set_reveal_duration (HdyFlap *self,
 }
 
 /**
- * hdy_flap_get_reveal_progress: (attributes org.gtk.Method.get_property=reveal-progress)
- * @self: a flap
+ * hdy_flap_get_reveal_progress:
+ * @self: a #HdyFlap
  *
- * Gets the current reveal transition progress for @self.
+ * Gets the current reveal transition progress for @self. 0 means fully hidden,
+ * 1 means fully revealed. See #HdyFlap:reveal-flap.
  *
  * Returns: the current reveal progress for @self
  *
@@ -2334,10 +2284,10 @@ hdy_flap_get_reveal_progress (HdyFlap *self)
 }
 
 /**
- * hdy_flap_get_fold_policy: (attributes org.gtk.Method.get_property=fold-policy)
- * @self: a flap
+ * hdy_flap_get_fold_policy:
+ * @self: a #HdyFlap
  *
- * Gets the current fold policy of @self.
+ * Gets the current fold policy of @self. See hdy_flap_set_fold_policy().
  *
  * Returns: the current fold policy of @self
  *
@@ -2352,11 +2302,12 @@ hdy_flap_get_fold_policy (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_fold_policy: (attributes org.gtk.Method.set_property=fold-policy)
- * @self: a flap
- * @policy: a fold policy
+ * hdy_flap_set_fold_policy:
+ * @self: a #HdyFlap
+ * @policy: Fold policy
  *
- * Sets the current fold policy for @self.
+ * Sets the current fold policy for @self. See #HdyFlapFoldPolicy for available
+ * policies.
  *
  * Since: 1.2
  */
@@ -2393,12 +2344,13 @@ hdy_flap_set_fold_policy (HdyFlap           *self,
 }
 
 /**
- * hdy_flap_get_fold_duration: (attributes org.gtk.Method.get_property=fold-duration)
- * @self: a flap
+ * hdy_flap_get_fold_duration:
+ * @self: a #HdyFlap
  *
- * Gets the amount of time that fold transitions will take.
+ * Returns the amount of time (in milliseconds) that fold transitions in @self
+ * will take.
  *
- * Returns: the fold transition duration, in milliseconds
+ * Returns: the fold transition duration
  *
  * Since: 1.2
  */
@@ -2411,11 +2363,11 @@ hdy_flap_get_fold_duration (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_fold_duration: (attributes org.gtk.Method.set_property=fold-duration)
- * @self: a flap
+ * hdy_flap_set_fold_duration:
+ * @self: a #HdyFlap
  * @duration: the new duration, in milliseconds
  *
- * Sets the duration that fold transitions will take.
+ * Sets the duration that fold transitions in @self will take.
  *
  * Since: 1.2
  */
@@ -2434,12 +2386,14 @@ hdy_flap_set_fold_duration (HdyFlap *self,
 }
 
 /**
- * hdy_flap_get_folded: (attributes org.gtk.Method.get_property=folded)
- * @self: a flap
+ * hdy_flap_get_folded:
+ * @self: a #HdyFlap
  *
  * Gets whether @self is currently folded.
  *
- * Returns: `TRUE` if @self is currently folded
+ * See #HdyFlap:fold-policy.
+ *
+ * Returns: %TRUE if @self is currently folded, %FALSE otherwise
  *
  * Since: 1.2
  */
@@ -2452,12 +2406,12 @@ hdy_flap_get_folded (HdyFlap *self)
 }
 
 /**
- * hdy_flap_get_locked: (attributes org.gtk.Method.get_property=locked)
- * @self: a flap
+ * hdy_flap_get_locked:
+ * @self: a #HdyFlap
  *
  * Gets whether @self is locked.
  *
- * Returns: whether @self is locked
+ * Returns: %TRUE if @self is locked, %FALSE otherwise
  *
  * Since: 1.2
  */
@@ -2470,15 +2424,15 @@ hdy_flap_get_locked (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_locked: (attributes org.gtk.Method.set_property=locked)
- * @self: a flap
+ * hdy_flap_set_locked:
+ * @self: a #HdyFlap
  * @locked: the new value
  *
  * Sets whether @self is locked.
  *
- * If `FALSE`, folding @self when the flap is revealed automatically closes it,
- * and unfolding it when the flap is not revealed opens it. If `TRUE`,
- * [property@Flap:reveal-flap] value never changes on its own.
+ * If %FALSE, folding @self when the flap is revealed automatically closes it,
+ * and unfolding it when the flap is not revealed opens it. If %TRUE,
+ * #HdyFlap:reveal-flap value never changes on its own.
  *
  * Since: 1.2
  */
@@ -2499,10 +2453,11 @@ hdy_flap_set_locked (HdyFlap  *self,
 }
 
 /**
- * hdy_flap_get_transition_type: (attributes org.gtk.Method.get_property=transition-type)
- * @self: a flap
+ * hdy_flap_get_transition_type:
+ * @self: a #HdyFlap
  *
- * Gets the type of animation used for reveal and fold transitions in @self.
+ * Gets the type of animation that will be used for reveal and fold transitions
+ * in @self.
  *
  * Returns: the current transition type of @self
  *
@@ -2517,11 +2472,16 @@ hdy_flap_get_transition_type (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_transition_type: (attributes org.gtk.Method.set_property=transition-type)
- * @self: a flap
+ * hdy_flap_set_transition_type:
+ * @self: a #HdyFlap
  * @transition_type: the new transition type
  *
- * Sets the type of animation used for reveal and fold transitions in @self.
+ * Sets the type of animation that will be used for reveal and fold transitions
+ * in @self.
+ *
+ * #HdyFlap:flap is transparent by default, which means the content will be seen
+ * through it with %HDY_FLAP_TRANSITION_TYPE_OVER transitions; add the
+ * .background style class to it if this is unwanted.
  *
  * Since: 1.2
  */
@@ -2546,12 +2506,12 @@ hdy_flap_set_transition_type (HdyFlap               *self,
 }
 
 /**
- * hdy_flap_get_modal: (attributes org.gtk.Method.get_property=modal)
- * @self: a flap
+ * hdy_flap_get_modal:
+ * @self: a #HdyFlap
  *
- * Gets whether the @self is modal.
+ * Gets whether the @self is modal. See hdy_flap_set_modal().
  *
- * Returns: whether @self is modal
+ * Returns: %TRUE if @self is modal
  *
  * Since: 1.2
  */
@@ -2564,15 +2524,15 @@ hdy_flap_get_modal (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_modal: (attributes org.gtk.Method.set_property=modal)
- * @self: a flap
- * @modal: whether @self can be closed with a click
+ * hdy_flap_set_modal:
+ * @self: a #HdyFlap
+ * @modal: Whether @self can be closed with a click
  *
  * Sets whether the @self can be closed with a click.
  *
- * If @modal is `TRUE`, clicking the content widget while flap is revealed, or
- * pressing the <kbd>Esc</kbd> key, will close the flap. If `FALSE`, clicks are
- * passed through to the content widget.
+ * If @modal is %TRUE, clicking the content widget while flap is revealed, or
+ * pressing Escape key, will close the flap. If %FALSE, clicks are passed
+ * through to the content widget.
  *
  * Since: 1.2
  */
@@ -2600,12 +2560,12 @@ hdy_flap_set_modal (HdyFlap  *self,
 }
 
 /**
- * hdy_flap_get_swipe_to_open: (attributes org.gtk.Method.get_property=swipe-to-open)
- * @self: a flap
+ * hdy_flap_get_swipe_to_open:
+ * @self: a #HdyFlap
  *
  * Gets whether @self can be opened with a swipe gesture.
  *
- * Returns: `TRUE` if @self can be opened with a swipe gesture
+ * Returns: %TRUE if @self can be opened with a swipe gesture
  *
  * Since: 1.2
  */
@@ -2618,14 +2578,13 @@ hdy_flap_get_swipe_to_open (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_swipe_to_open: (attributes org.gtk.Method.set_property=swipe-to-open)
- * @self: a flap
- * @swipe_to_open: whether @self can be opened with a swipe gesture
+ * hdy_flap_set_swipe_to_open:
+ * @self: a #HdyFlap
+ * @swipe_to_open: Whether @self can be opened with a swipe gesture
  *
  * Sets whether @self can be opened with a swipe gesture.
  *
- * The area that can be swiped depends on the [property@Flap:transition-type]
- * value.
+ * The area that can be swiped depends on the #HdyFlap:transition-type value.
  *
  * Since: 1.2
  */
@@ -2648,12 +2607,12 @@ hdy_flap_set_swipe_to_open (HdyFlap  *self,
 }
 
 /**
- * hdy_flap_get_swipe_to_close: (attributes org.gtk.Method.get_property=swipe-to-close)
- * @self: a flap
+ * hdy_flap_get_swipe_to_close:
+ * @self: a #HdyFlap
  *
  * Gets whether @self can be closed with a swipe gesture.
  *
- * Returns: `TRUE` if @self can be closed with a swipe gesture
+ * Returns: %TRUE if @self can be closed with a swipe gesture
  *
  * Since: 1.2
  */
@@ -2666,13 +2625,13 @@ hdy_flap_get_swipe_to_close (HdyFlap *self)
 }
 
 /**
- * hdy_flap_set_swipe_to_close: (attributes org.gtk.Method.set_property=swipe-to-close)
- * @self: a flap
- * @swipe_to_close: whether @self can be closed with a swipe gesture
+ * hdy_flap_set_swipe_to_close:
+ * @self: a #HdyFlap
+ * @swipe_to_close: Whether @self can be closed with a swipe gesture
  *
  * Sets whether @self can be closed with a swipe gesture.
  *
- * The area that can be swiped depends on the [property@Flap:transition-type] value.
+ * The area that can be swiped depends on the #HdyFlap:transition-type value.
  *
  * Since: 1.2
  */
