@@ -10,7 +10,7 @@
 
 #include "hdy-leaflet.h"
 #include "hdy-stackable-box-private.h"
-#include "hdy-swipeable-private.h"
+#include "hdy-swipeable.h"
 
 /**
  * SECTION:hdy-leaflet
@@ -22,8 +22,21 @@
  * the two modes.
  *
  * When there is enough space the children are displayed side by side, otherwise
- * only one is displayed. The threshold is dictated by the preferred minimum
- * sizes of the children.
+ * only one is displayed and the leaflet is said to be “folded”.
+ * The threshold is dictated by the preferred minimum sizes of the children.
+ * When a leaflet is folded, the children can be navigated using swipe gestures.
+ *
+ * The “over” and “under” stack the children one on top of the other, while the
+ * “slide” transition puts the children side by side. While navigating to a
+ * child on the side or below can be performed by swiping the current child
+ * away, navigating to an upper child requires dragging it from the edge where
+ * it resides. This doesn't affect non-dragging swipes.
+ *
+ * The “over” and “under” transitions can draw their shadow on top of the
+ * window's transparent areas, like the rounded corners. This is a side-effect
+ * of allowing shadows to be drawn on top of OpenGL areas. It can be mitigated
+ * by using #HdyWindow or #HdyApplicationWindow as they will crop anything drawn
+ * beyond the rounded corners.
  *
  * # CSS nodes
  *
@@ -71,7 +84,7 @@ enum {
 enum {
   CHILD_PROP_0,
   CHILD_PROP_NAME,
-  CHILD_PROP_ALLOW_VISIBLE,
+  CHILD_PROP_NAVIGATABLE,
   LAST_CHILD_PROP,
 };
 
@@ -409,7 +422,7 @@ hdy_leaflet_set_interpolate_size (HdyLeaflet *self,
  * hdy_leaflet_get_interpolate_size:
  * @self: a #HdyLeaflet
  *
- * Returns wether the #HdyLeaflet is set up to interpolate between
+ * Returns whether the #HdyLeaflet is set up to interpolate between
  * the sizes of children on page switch.
  *
  * Returns: %TRUE if child sizes are interpolated
@@ -428,7 +441,7 @@ hdy_leaflet_get_interpolate_size (HdyLeaflet *self)
  * @can_swipe_back: the new value
  *
  * Sets whether or not @self allows switching to the previous child that has
- * 'allow-visible' child property set to %TRUE via a swipe gesture
+ * 'navigatable' child property set to %TRUE via a swipe gesture
  *
  * Since: 0.0.12
  */
@@ -465,7 +478,7 @@ hdy_leaflet_get_can_swipe_back (HdyLeaflet *self)
  * @can_swipe_forward: the new value
  *
  * Sets whether or not @self allows switching to the next child that has
- * 'allow-visible' child property set to %TRUE via a swipe gesture.
+ * 'navigatable' child property set to %TRUE via a swipe gesture.
  *
  * Since: 0.0.12
  */
@@ -501,7 +514,7 @@ hdy_leaflet_get_can_swipe_forward (HdyLeaflet *self)
  * @self: a #HdyLeaflet
  * @direction: the direction
  *
- * Gets the previous or next child that doesn't have 'allow-visible' child
+ * Gets the previous or next child that doesn't have 'navigatable' child
  * property set to %FALSE, or %NULL if it doesn't exist. This will be the same
  * widget hdy_leaflet_navigate() will navigate to.
  *
@@ -524,9 +537,9 @@ hdy_leaflet_get_adjacent_child (HdyLeaflet             *self,
  * @self: a #HdyLeaflet
  * @direction: the direction
  *
- * Switches to the previous or next child that doesn't have 'allow-visible'
- * child property set to %FALSE, similar to performing a swipe gesture to go
- * in @direction.
+ * Switches to the previous or next child that doesn't have 'navigatable' child
+ * property set to %FALSE, similar to performing a swipe gesture to go in
+ * @direction.
  *
  * Returns: %TRUE if visible child was changed, %FALSE otherwise.
  *
@@ -541,6 +554,102 @@ hdy_leaflet_navigate (HdyLeaflet             *self,
   return hdy_stackable_box_navigate (HDY_GET_HELPER (self), direction);
 }
 
+/**
+ * hdy_leaflet_get_child_by_name:
+ * @self: a #HdyLeaflet
+ * @name: the name of the child to find
+ *
+ * Finds the child of @self with the name given as the argument. Returns %NULL
+ * if there is no child with this name.
+ *
+ * Returns: (transfer none) (nullable): the requested child of @self
+ *
+ * Since: 1.0
+ */
+GtkWidget *
+hdy_leaflet_get_child_by_name (HdyLeaflet  *self,
+                               const gchar *name)
+{
+  g_return_val_if_fail (HDY_IS_LEAFLET (self), NULL);
+
+  return hdy_stackable_box_get_child_by_name (HDY_GET_HELPER (self), name);
+}
+
+/**
+ * hdy_leaflet_prepend:
+ * @self: a #HdyLeaflet
+ * @child: the #GtkWidget to prepend
+ *
+ * Inserts @child at the first position in @self.
+ *
+ * Since: 1.2
+ */
+void
+hdy_leaflet_prepend (HdyLeaflet *self,
+                     GtkWidget  *child)
+{
+  g_return_if_fail (HDY_IS_LEAFLET (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
+  hdy_stackable_box_prepend (HDY_GET_HELPER (self), child);
+}
+
+/**
+ * hdy_leaflet_insert_child_after:
+ * @self: a #HdyLeaflet
+ * @child: the #GtkWidget to insert
+ * @sibling: (nullable): the sibling after which to insert @child
+ *
+ * Inserts @child in the position after @sibling in the list of children.
+ * If @sibling is %NULL, insert @child at the first position.
+ *
+ * Since: 1.2
+ */
+void
+hdy_leaflet_insert_child_after (HdyLeaflet *self,
+                                GtkWidget  *child,
+                                GtkWidget  *sibling)
+{
+  g_return_if_fail (HDY_IS_LEAFLET (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (sibling == NULL || GTK_IS_WIDGET (sibling));
+
+  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+  g_return_if_fail (sibling == NULL || gtk_widget_get_parent (sibling) == GTK_WIDGET (self));
+
+  hdy_stackable_box_insert_child_after (HDY_GET_HELPER (self), child, sibling);
+}
+
+/**
+ * hdy_leaflet_reorder_child_after:
+ * @self: a #HdyLeaflet
+ * @child: the #GtkWidget to move, must be a child of @self
+ * @sibling: (nullable): the sibling to move @child after, or %NULL
+ *
+ * Moves @child to the position after @sibling in the list of children.
+ * If @sibling is %NULL, move @child to the first position.
+ *
+ * Since: 1.2
+ */
+void
+hdy_leaflet_reorder_child_after (HdyLeaflet *self,
+                                 GtkWidget  *child,
+                                 GtkWidget  *sibling)
+{
+  g_return_if_fail (HDY_IS_LEAFLET (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (sibling == NULL || GTK_IS_WIDGET (sibling));
+
+  g_return_if_fail (gtk_widget_get_parent (child) == GTK_WIDGET (self));
+  g_return_if_fail (sibling == NULL || gtk_widget_get_parent (sibling) == GTK_WIDGET (self));
+
+  if (child == sibling)
+    return;
+
+  hdy_stackable_box_reorder_child_after (HDY_GET_HELPER (self), child, sibling);
+}
+
 /* This private method is prefixed by the call name because it will be a virtual
  * method in GTK 4.
  */
@@ -776,8 +885,8 @@ hdy_leaflet_get_child_property (GtkContainer *container,
     g_value_set_string (value, hdy_stackable_box_get_child_name (HDY_GET_HELPER (container), widget));
     break;
 
-  case CHILD_PROP_ALLOW_VISIBLE:
-    g_value_set_boolean (value, hdy_stackable_box_get_child_allow_visible (HDY_GET_HELPER (container), widget));
+  case CHILD_PROP_NAVIGATABLE:
+    g_value_set_boolean (value, hdy_stackable_box_get_child_navigatable (HDY_GET_HELPER (container), widget));
     break;
 
   default:
@@ -799,8 +908,8 @@ hdy_leaflet_set_child_property (GtkContainer *container,
     gtk_container_child_notify_by_pspec (container, widget, pspec);
     break;
 
-  case CHILD_PROP_ALLOW_VISIBLE:
-    hdy_stackable_box_set_child_allow_visible (HDY_GET_HELPER (container), widget, g_value_get_boolean (value));
+  case CHILD_PROP_NAVIGATABLE:
+    hdy_stackable_box_set_child_navigatable (HDY_GET_HELPER (container), widget, g_value_get_boolean (value));
     gtk_container_child_notify_by_pspec (container, widget, pspec);
     break;
 
@@ -842,27 +951,10 @@ hdy_leaflet_switch_child (HdySwipeable *swipeable,
   hdy_stackable_box_switch_child (HDY_GET_HELPER (swipeable), index, duration);
 }
 
-static void
-hdy_leaflet_begin_swipe (HdySwipeable           *swipeable,
-                         HdyNavigationDirection  direction,
-                         gboolean                direct)
+static HdySwipeTracker *
+hdy_leaflet_get_swipe_tracker (HdySwipeable *swipeable)
 {
-  hdy_stackable_box_begin_swipe (HDY_GET_HELPER (swipeable), direction, direct);
-}
-
-static void
-hdy_leaflet_update_swipe (HdySwipeable *swipeable,
-                          gdouble       value)
-{
-  hdy_stackable_box_update_swipe (HDY_GET_HELPER (swipeable), value);
-}
-
-static void
-hdy_leaflet_end_swipe (HdySwipeable    *swipeable,
-                       gint64           duration,
-                       gdouble          to)
-{
-  hdy_stackable_box_end_swipe (HDY_GET_HELPER (swipeable), duration, to);
+  return hdy_stackable_box_get_swipe_tracker (HDY_GET_HELPER (swipeable));
 }
 
 static gdouble
@@ -871,12 +963,11 @@ hdy_leaflet_get_distance (HdySwipeable *swipeable)
   return hdy_stackable_box_get_distance (HDY_GET_HELPER (swipeable));
 }
 
-static void
-hdy_leaflet_get_range (HdySwipeable *swipeable,
-                       gdouble      *lower,
-                       gdouble      *upper)
+static gdouble *
+hdy_leaflet_get_snap_points (HdySwipeable *swipeable,
+                             gint         *n_snap_points)
 {
-  hdy_stackable_box_get_range (HDY_GET_HELPER (swipeable), lower, upper);
+  return hdy_stackable_box_get_snap_points (HDY_GET_HELPER (swipeable), n_snap_points);
 }
 
 static gdouble
@@ -891,6 +982,15 @@ hdy_leaflet_get_cancel_progress (HdySwipeable *swipeable)
   return hdy_stackable_box_get_cancel_progress (HDY_GET_HELPER (swipeable));
 }
 
+static void
+hdy_leaflet_get_swipe_area (HdySwipeable           *swipeable,
+                            HdyNavigationDirection  navigation_direction,
+                            gboolean                is_drag,
+                            GdkRectangle           *rect)
+{
+  hdy_stackable_box_get_swipe_area (HDY_GET_HELPER (swipeable), navigation_direction, is_drag, rect);
+}
+
 static void
 hdy_leaflet_class_init (HdyLeafletClass *klass)
 {
@@ -1052,8 +1152,8 @@ hdy_leaflet_class_init (HdyLeafletClass *klass)
   /**
    * HdyLeaflet:can-swipe-back:
    *
-   * Whether or not @self allows switching to the previous child that has
-   * 'allow-visible' child property set to %TRUE via a swipe gesture.
+   * Whether or not the leaflet allows switching to the previous child that has
+   * 'navigatable' child property set to %TRUE via a swipe gesture.
    *
    * Since: 0.0.12
    */
@@ -1067,8 +1167,8 @@ hdy_leaflet_class_init (HdyLeafletClass *klass)
   /**
    * HdyLeaflet:can-swipe-forward:
    *
-   * Whether or not @self allows switching to the next child that has
-   * 'allow-visible' child property set to %TRUE via a swipe gesture.
+   * Whether or not the leaflet allows switching to the next child that has
+   * 'navigatable' child property set to %TRUE via a swipe gesture.
    *
    * Since: 0.0.12
    */
@@ -1089,19 +1189,20 @@ hdy_leaflet_class_init (HdyLeafletClass *klass)
                          G_PARAM_READWRITE);
 
   /**
-   * HdyLeaflet:allow-visible:
+   * HdyLeaflet:navigatable:
    *
-   * Whether the child can be visible when folded. This can be used used in
-   * conjunction with #HdyLeaflet:can-swipe-back or
-   * #HdyLeaflet:can-swipe-forward to prevent switching to widgets like
-   * separators.
+   * Whether the child can be navigated to when folded.
+   * If %FALSE, the child will be ignored by hdy_leaflet_get_adjacent_child(),
+   * hdy_leaflet_navigate(), and swipe gestures.
    *
-   * Since: 0.0.12
+   * This can be used used to prevent switching to widgets like separators.
+   *
+   * Since: 1.0
    */
-  child_props[CHILD_PROP_ALLOW_VISIBLE] =
-    g_param_spec_boolean ("allow-visible",
-                          _("Allow visible"),
-                          _("Whether the child can be visible in folded mode"),
+  child_props[CHILD_PROP_NAVIGATABLE] =
+    g_param_spec_boolean ("navigatable",
+                          _("Navigatable"),
+                          _("Whether the child can be navigated to"),
                           TRUE,
                           G_PARAM_READWRITE);
 
@@ -1174,11 +1275,10 @@ static void
 hdy_leaflet_swipeable_init (HdySwipeableInterface *iface)
 {
   iface->switch_child = hdy_leaflet_switch_child;
-  iface->begin_swipe = hdy_leaflet_begin_swipe;
-  iface->update_swipe = hdy_leaflet_update_swipe;
-  iface->end_swipe = hdy_leaflet_end_swipe;
+  iface->get_swipe_tracker = hdy_leaflet_get_swipe_tracker;
   iface->get_distance = hdy_leaflet_get_distance;
-  iface->get_range = hdy_leaflet_get_range;
+  iface->get_snap_points = hdy_leaflet_get_snap_points;
   iface->get_progress = hdy_leaflet_get_progress;
   iface->get_cancel_progress = hdy_leaflet_get_cancel_progress;
+  iface->get_swipe_area = hdy_leaflet_get_swipe_area;
 }
